#ifndef WINDOWSSEXCEPTION_H_
#define WINDOWSEXCEPTION_H_

#include "wruntime_error.h"

// Exception class that includes a message based on an error code.
class WindowsException : public wruntime_error
{
public:
	// Create a new instance using the specified base message and error code. The message belonging to the
	// error code is appended to the base message.
	WindowsException(const std::wstring &baseMessage, DWORD error) : wruntime_error(GetErrorMessage(baseMessage, error))
	{
	}

	virtual ~WindowsException()
	{
	}

private:
	// Get the error message for a specific error code.
	static std::wstring GetErrorMessage(const std::wstring &baseMessage, DWORD error)
	{
		LPWSTR message = NULL;
		if( FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_ALLOCATE_BUFFER, NULL, error, 0, reinterpret_cast<LPTSTR>(&message), 0, NULL) == 0 )
		{
			std::wostringstream ss;
			ss << baseMessage << L"An unknown error occurred, error number 0x" << std::hex << error << std::flush;
			return ss.str();
		}
		else
		{
			std::wostringstream ss;
			ss << baseMessage << message << std::flush;
			LocalFree(message);
			return ss.str();
		}
	}
};

#endif