#ifndef WRUNTIME_ERROR
#define WRUNTIME_ERROR

// Class that adds wstring support to runtime_error
class wruntime_error : public std::runtime_error
{
public:
	wruntime_error(const std::wstring &message) : std::runtime_error(ToNarrowString(message)), _message(message)
	{
	}

	wruntime_error(const wruntime_error &right) : std::runtime_error(right.what()), _message(right.ErrorMessage())
	{
	}

	wruntime_error& operator=(const wruntime_error &right)
	{
		std::runtime_error::operator=(right);
		_message = right.ErrorMessage();
	}
	
	virtual ~wruntime_error()
	{
	}

	std::wstring ErrorMessage() const
	{
		return _message;
	}
private:
	static std::string ToNarrowString(const std::wstring &s)
	{
		const wchar_t *str = s.c_str();
		int len = static_cast<int>(s.length());

		// figure out how many narrow characters we are going to get 
		int chars = WideCharToMultiByte(CP_ACP, 0, str, len, NULL, 0, NULL, NULL); 

		if( chars == 0 )
			return "";

		// convert the wide string to a narrow string
		char *buf = new char[chars];
		WideCharToMultiByte(CP_ACP, 0, str, len, buf, chars, NULL, NULL); 

		std::string result = buf;
		delete[] buf;
		return result; 
	}

	std::wstring _message;
};

#endif